<?php
/**
* This is main class of module. 
* 
* @author    Globo Jsc <contact@globosoftware.net>
* @copyright 2016 Globo., Jsc
* @link      http://www.globosoftware.net/
* @license   please read license in file license.txt
*/

/**
 * @since   1.6.0.5
 */
if (!defined('_PS_VERSION_'))
    exit;

class G_cmsasHomepage extends Module
{
    protected $_html = '';
    
    public function __construct()
    {   
        $this->name = 'g_cmsashomepage';
        $this->tab = 'front_office_features';
        $this->version = '1.0.0';
        $this->author = 'Globo Jsc';
        $this->need_instance = 0;
        $this->secure_key = Tools::encrypt($this->name);
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Set cms page as home page');
        $this->description = $this->l('The module allows you to set a CMS page as Home page. And you may use old home page as another page. E.g: Shop page');
        $this->confirmUninstall = $this->l('Are you sure you want to uninstall?');
        $this->ps_versions_compliancy = array('min' => '1.6.0.5', 'max' => _PS_VERSION_);
        $this->module_key = '42ee7e9ea418ee927f439805bcee768c';
    }
    
    /**
	 * @see Module::install()
	 */
    public function install()
    {
        /* Adds Module */
        if(parent::install()
        && $this->registerHook('moduleRoutes') 
        && $this->registerHook('actionAdminControllerSetMedia'))
        {
            $res = $this->addPageModule();
        	return (bool)$res;   
        }
        return false;
    }
    
    public function addPageModule(){
        $value = Db::getInstance()->getValue('SELECT MAX(id_meta) FROM '._DB_PREFIX_.'meta');
        $id_meta = $value + 1;
        $page_name = 'module-g_cmsashomepage-oldhome';
        $configurable  = 1;
        $title = 'Shop';
        $description = '';
        $keywords = '';
        $url_rewite = 'shop';
        
        Db::getInstance()->execute('INSERT INTO `'._DB_PREFIX_.'meta` (`id_meta`, `page`, `configurable`) VALUES ('.(int)$id_meta.', \''.$page_name.'\', '.(int)$configurable.') ON DUPLICATE KEY UPDATE `id_meta` = '.(int)$id_meta);
        
        $languages = Language::getLanguages(false);

        foreach ( $languages as $language )
        {
            $id_shop = (int)$language['id_shop'];
            $id_lang = (int)$language['id_lang'];
            Db::getInstance()->execute('INSERT INTO `'._DB_PREFIX_.'meta_lang` (`id_meta`, `id_shop`, `id_lang`, `title`, `description`, `keywords`, `url_rewrite`) VALUES ('. (int)$id_meta .', '. (int)$id_shop .', '. (int)$id_lang .', \''. $title .'\', \''. $description.'\', \''. $keywords.'\', \''. $url_rewite.'\') ON DUPLICATE KEY UPDATE `id_meta` = '.(int)$id_meta);
        }
        
        return true;
    }
    
    public function hookActionAdminControllerSetMedia()
    {
        $this->context->controller->addJS($this->_path.'views/js/admin/custom.js');
    }
    
    public function hookModuleRoutes($route = '', $detail = array())
	{   
        if(Module::isInstalled('g_cmsashomepage'))
        {
            if (Module::isEnabled('g_cmsashomepage'))
            {
                $routes = array();
                $g_conf = array();
                $g_conf = $this->getConfigFieldsValues();
                            
                if($g_conf['cms_page_stt'] == 1)
                {
                    $g_confhomepage = array();
                    $g_confhomepage = $this->getAddConfigFieldsValues();
                    $active_homepage = $g_confhomepage['homepage_stt']; 
                    
                    if( $active_homepage == 1)
                    {
                        $config = $this->getInformationOldHomepage();
                        foreach($config as $cf)
                        {
                            $url_oldhome = $cf['url_rewrite'];
                        }
                        
                        $routes['globocmsashomepage'] = array(
                			'controller' => 'oldhome',
                			'rule' => $url_oldhome,
                			'keywords' => array(
                				'id' => array('regexp' => '[0-9]+', 'param' => 'id'),
                				'rewrite' => array('regexp' => '[_a-zA-Z0-9-\pL]*'),
                			),
                			'params' => array(
                				'fc' => 'module',
                				'module' => 'g_cmsashomepage',
                			)
                        );   
                    }
                    return $routes;
                }   
            } else {
                return true;
            }
        } else {
            return true;          
        }                                
	}
    
    /**
	 * @see Module::uninstall()
	 */
    public function uninstall()
    {
        /* Deletes Module */
		if (parent::uninstall() 
        && $this->unregisterHook('moduleRoutes') 
        && $this->unregisterHook('actionAdminControllerSetMedia'))
		{
			/* Unsets configuration */
			$res = Configuration::deleteByName('cms_page_name');
            $res &= Configuration::deleteByName('cms_page_stt');
            $res &= Configuration::deleteByName('homepage_stt');
            
			return (bool)$res;
		}
		return false;
    }
    
    public function getContent()
    {
        $g_conf = array();
        $g_conf = $this->getConfigFieldsValues();
        
        $shop_context = Shop::getContext();
        $shops = Shop::getContextListShopID();
        $shop_groups_list = array();
        $errors = array();
        if (Tools::isSubmit('submitSetCmsPage'))
		{
            if (isset($errors) && count($errors))
            {
                $this->_html .= $this->displayError(implode('<br />', $errors));
            }
            else
            { 
                foreach ($shops as $shop_id)
    			{
    				$shop_group_id = (int)Shop::getGroupFromShop((int)$shop_id, true);
    				if (!in_array($shop_group_id, $shop_groups_list))
    					$shop_groups_list[] = $shop_group_id;
    
    				$res  = Configuration::updateValue('cms_page_name',Tools::getValue('cms_page_name'), false, $shop_group_id, (int)$shop_id);
                    $res  &= Configuration::updateValue('cms_page_stt',Tools::getValue('cms_page_stt'), false, $shop_group_id, (int)$shop_id);
    			}
                switch ($shop_context)
    			{
    				case Shop::CONTEXT_ALL:
    					$res = Configuration::updateValue('cms_page_name', Tools::getValue('cms_page_name'));
                        $res &= Configuration::updateValue('cms_page_stt', Tools::getValue('cms_page_stt'));
    					if (count($shop_groups_list))
    					{
    						foreach ($shop_groups_list as $shop_group_id)
    						{
    							$res = Configuration::updateValue('cms_page_name', Tools::getValue('cms_page_name'), false, (int)$shop_group_id);
                                $res &= Configuration::updateValue('cms_page_stt', Tools::getValue('cms_page_stt'), false, (int)$shop_group_id);
    						}
    					}
    					break;
    				case Shop::CONTEXT_GROUP:
    					if (count($shop_groups_list))
    					{
    						foreach ($shop_groups_list as $shop_group_id)
    						{
                                $res = Configuration::updateValue('cms_page_name', Tools::getValue('cms_page_name'), false, (int)$shop_group_id);
                                $res &= Configuration::updateValue('cms_page_stt', Tools::getValue('cms_page_stt'), false, (int)$shop_group_id);
    						}
    					}
    					break;
    			}
                $this->_html .= $this->displayConfirmation($this->l('The settings are saved successful.'));
            }
            if (!$res)
                return false;
        }
        elseif (Tools::isSubmit('submitActiveOldHomepage'))
		{   
            if (isset($errors) && count($errors))
            {
                $this->_html .= $this->displayError(implode('<br />', $errors));
            }
            else
            { 
                foreach ($shops as $shop_id)
    			{
    				$shop_group_id = (int)Shop::getGroupFromShop((int)$shop_id, true);
    				if (!in_array($shop_group_id, $shop_groups_list))
    					$shop_groups_list[] = $shop_group_id;
    
    				$res  = Configuration::updateValue('homepage_stt',Tools::getValue('homepage_stt'), false, $shop_group_id, (int)$shop_id);
                }
                switch ($shop_context)
    			{
    				case Shop::CONTEXT_ALL:
    					$res = Configuration::updateValue('homepage_stt', Tools::getValue('homepage_stt'));
    					if (count($shop_groups_list))
    					{
    						foreach ($shop_groups_list as $shop_group_id)
    						{
    							$res = Configuration::updateValue('homepage_stt', Tools::getValue('homepage_stt'), false, (int)$shop_group_id);
                            }
    					}
    					break;
    				case Shop::CONTEXT_GROUP:
    					if (count($shop_groups_list))
    					{
    						foreach ($shop_groups_list as $shop_group_id)
    						{
                                $res = Configuration::updateValue('homepage_stt', Tools::getValue('homepage_stt'), false, (int)$shop_group_id);
                            }
    					}
    					break;
    			}
                $this->_html .= $this->displayConfirmation($this->l('The settings are saved successful.'));
            }
        }
        
        if($g_conf['cms_page_stt'] == 1){
            return $this->_html.$this->renderForm().$this->renderAddForm();
        } else {
            return $this->_html.$this->renderForm();
        }
    }

    public function renderForm()
	{
        $id_lang = $this->context->language->id;
        $results = CMSCore::listCms($id_lang);
        $cms_array = array();
        foreach($results as $result)
        {
            $cms_array[] = array(
                'id_cms'=>(int)$result['id_cms'],
                'meta_title'=>$result['meta_title']
            );
        }
        
		$fields_form = array(
			'form' => array(
				'legend' => array(
					'title' => $this->l('Settings'),
					'icon' => 'icon-cogs'
				),
				'input' => array(
                    array(
						'type' => 'switch',
						'label' => $this->l('Active the module ?'),
						'name' => 'cms_page_stt',
                        'values' => array(
							array(
								'id' => 'gcmshome_status_on',
								'value' => 1,
								'label' => $this->l('Enable')
							),
							array(
								'id' => 'gcmshome_status_off',
								'value' => 0,
								'label' => $this->l('Disable')
							)
						),
					),
                    array( 
                        'type' => 'select',
    					'label' => $this->l('Select a CMS page'),
    					'name' => 'cms_page_name',
                        'desc' => $this->l('The CMS page will be displayed as a new home page'),
    					'options' => array(
    						'query' => $cms_array,
    						'id' => 'id_cms',
    						'name' => 'meta_title',                            
					       ),
                    ),
				),
                'submit' => array(
					'title' => $this->l('Save'),
				)
			),
		);

		$helper = new HelperForm();
		$helper->show_toolbar = false;
		$helper->table =  $this->table;
		$lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
		$helper->default_form_language = $lang->id;
		$helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
		$helper->identifier = $this->identifier;
		$helper->submit_action = 'submitSetCmsPage';
		$helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
		$helper->token = Tools::getAdminTokenLite('AdminModules');
		$helper->tpl_vars = array(
			'fields_value' => $this->getConfigFieldsValues(),
			'languages' => $this->context->controller->getLanguages(),
			'id_language' => $this->context->language->id
		);
		return $helper->generateForm(array($fields_form));
	}
    
    public function renderAddForm()
	{
        $html = '';
        $conf_url_fr = Configuration::get('PS_REWRITING_SETTINGS');
        $g_confhomepage = $this->getAddConfigFieldsValues();
        $result = $this->getInformationOldHomepage();
        foreach($result as $rs)
        {
            $path_new_url_homepage = $rs['url_rewrite'];
        }
        
        $url_old_homepage = _PS_BASE_URL_.__PS_BASE_URI__.$path_new_url_homepage;
        $url_old_homepage_ = _PS_BASE_URL_.__PS_BASE_URI__.'index.php?fc=module&module=g_cmsashomepage&controller=oldhome';
        
        if($g_confhomepage['homepage_stt'] == 1 && $conf_url_fr == 1)
        {
            $html = $this->l('New URL of old homepage : ').'<a href="'.$url_old_homepage.'" target = "_blank">'.$url_old_homepage.'</a>';
        } elseif ($g_confhomepage['homepage_stt'] == 1 && $conf_url_fr == 0) {
            $html = $this->l('New URL of old homepage : ').'<a href="'.$url_old_homepage_.'" target = "_blank">'.$url_old_homepage_.'</a>';
        }
                
		$fields_form = array(
			'form' => array(
				'legend' => array(
					'title' => $this->l('Set old home page as another page'),
					'icon' => 'icon-cogs'
				),
				'input' => array(
                    array(
						'type' => 'switch',
						'label' => $this->l('Display old home page?'),
                        'desc' => $this->l('Select YES if you want to use old home page as another page. E.g: a Shop page.').'<br />'.$html,
						'name' => 'homepage_stt',
                        'values' => array(
							array(
								'id' => 'homepage_status_on',
								'value' => 1,
								'label' => $this->l('Enable')
							),
							array(
								'id' => 'homepage_status_off',
								'value' => 0,
								'label' => $this->l('Disable')
							)
						),
					),
				),
                'submit' => array(
					'title' => $this->l('Save'),
				)
			),
		);

		$helper = new HelperForm();
		$helper->show_toolbar = false;
		$helper->table =  $this->table;
		$lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
		$helper->default_form_language = $lang->id;
		$helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
		$helper->identifier = $this->identifier;
		$helper->submit_action = 'submitActiveOldHomepage';
		$helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
		$helper->token = Tools::getAdminTokenLite('AdminModules');
		$helper->tpl_vars = array(
			'fields_value' => $this->getAddConfigFieldsValues(),
			'languages' => $this->context->controller->getLanguages(),
			'id_language' => $this->context->language->id
		);
		return $helper->generateForm(array($fields_form));
	}
    
    public function getAddConfigFieldsValues()
    {
        $id_shop_group = (int)Shop::getContextShopGroupID();
		$id_shop = (int)Shop::getContextShopID();
        
        return array(
            'homepage_stt' => Tools::getValue('homepage_stt', Configuration::get('homepage_stt', null, $id_shop_group, $id_shop)),
        );
    }
    
    public function getConfigFieldsValues()
    {
        $id_shop_group = (int)Shop::getContextShopGroupID();
		$id_shop = (int)Shop::getContextShopID();
        return array(
            'cms_page_name' => Tools::getValue('cms_page_name', Configuration::get('cms_page_name', null, $id_shop_group, $id_shop)),
            'cms_page_stt' => Tools::getValue('cms_page_stt', Configuration::get('cms_page_stt', null, $id_shop_group, $id_shop)),
        );
    }
    
    public function getInformationOldHomepage()
    {
        $page_name = 'module-g_cmsashomepage-oldhome';
        $id_lang = $this->context->language->id;
        $results = Db::getInstance()->getValue('SELECT `id_meta` FROM `'._DB_PREFIX_.'meta` WHERE `page` = \''.$page_name.'\'');
        $result = Db::getInstance()->executeS('SELECT * FROM `'._DB_PREFIX_.'meta_lang` WHERE `id_meta` = '.(int)$results.' AND `id_lang` = '.$id_lang);
        
        return $result;
    }
}